import { FromSchema } from 'json-schema-to-ts';
import { GraphQLESLintRule } from '../../types.cjs';
import 'eslint';
import 'estree';
import 'graphql';
import 'graphql-config';
import '../../estree-converter/types.cjs';
import '../../siblings.cjs';
import '@graphql-tools/utils';

declare const schema: {
    readonly type: "array";
    readonly maxItems: 1;
    readonly items: {
        readonly type: "object";
        readonly additionalProperties: false;
        readonly minProperties: 1;
        readonly properties: {
            readonly withEdgeSuffix: {
                readonly type: "boolean";
                readonly default: true;
                readonly description: "Edge type name must end in \"Edge\".";
            };
            readonly shouldImplementNode: {
                readonly type: "boolean";
                readonly default: true;
                readonly description: "Edge type's field `node` must implement `Node` interface.";
            };
            readonly listTypeCanWrapOnlyEdgeType: {
                readonly type: "boolean";
                readonly default: true;
                readonly description: "A list type should only wrap an edge type.";
            };
        };
    };
};
type RuleOptions = FromSchema<typeof schema, {
    keepDefaultedPropertiesOptional: true;
}>;
declare const rule: GraphQLESLintRule<RuleOptions, true>;

export { type RuleOptions, rule };
